"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileProcessor = void 0;
const image_resizing_1 = require("../utils/image-resizing");
const is_image_util_1 = require("../utils/is-image.util");
const file_dto_1 = require("./implementations/file.dto");
class FileProcessor {
    options;
    constructor(options) {
        this.options = options;
    }
    static isFile(file) {
        return file && file instanceof file_dto_1.FileDto;
    }
    getBuffer() {
        if (!this.options.buffer) {
            throw new Error("Buffer is not defined");
        }
        return this.options.buffer;
    }
    getSize() {
        return this.getBuffer().length;
    }
    async getResized() {
        const config = this.getConfig();
        const isImage = (0, is_image_util_1.isImageMime)(this.options.mimeType);
        if (!isImage || (config && !config.resize)) {
            return null;
        }
        if (config && config.resize && Object.keys(config.resize).length) {
            const resizeKeys = Object.keys(config.resize);
            for (const key of resizeKeys) {
                const resize = config.resize[key];
                if (!resize || key !== this.options.fieldName) {
                    continue;
                }
                const { width, height, quality } = resize;
                if (!width || !height) {
                    throw new Error(`Resize width and height are required for ${key} resize`);
                }
                return await image_resizing_1.ImageResizing.resize(this.getBuffer(), {
                    width,
                    height,
                    quality: quality ?? 100,
                });
            }
        }
        return null;
    }
    getMetadata() {
        return {
            originalName: this.options.originalName,
            encoding: this.options.encoding,
            extension: this.options.extension,
            mimeType: this.options.mimeType,
        };
    }
    getExtension() {
        return this.options.extension;
    }
    getConfig() {
        return this.options.config;
    }
}
exports.FileProcessor = FileProcessor;
