"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormProcessor = void 0;
const busboy_1 = __importDefault(require("busboy"));
const append_field_1 = __importDefault(require("append-field"));
const file_type_1 = __importDefault(require("file-type"));
const file_processor_factory_1 = require("../processors/file-processor.factory");
const common_1 = require("@nestjs/common");
const crypto_1 = require("crypto");
const concat_stream_1 = __importDefault(require("concat-stream"));
class FormProcessor {
    request;
    options;
    instance;
    bodyFields = {};
    files = [];
    handleFunctionToResolve;
    handleFunctionToReject;
    fileStorePromises = [];
    constructor(request, options) {
        this.request = request;
        this.options = options;
        this.instance = (0, busboy_1.default)({
            headers: this.request.headers,
        });
        this.instance.on("field", this.onField.bind(this));
        this.instance.on("file", this.onFile.bind(this));
        this.instance.on("error", this.rejectBadRequest.bind(this));
        this.instance.on("finish", this.processFileOnFinish.bind(this));
    }
    handle() {
        return new Promise((resolve, reject) => {
            this.handleFunctionToReject = reject;
            this.handleFunctionToResolve = resolve;
            this.request.pipe(this.instance);
        });
    }
    onField(name, value) {
        (0, append_field_1.default)(this.bodyFields, name, value);
    }
    async onFile(fileName, stream, meta) {
        const { filename } = meta;
        if (!filename) {
            stream.resume();
            return;
        }
        const processFile = this.processFile(fileName, stream, meta)
            .then(file => {
            if (stream.truncated) {
                this.instance.emit("fileSize");
                return;
            }
            this.files.push(file);
            (0, append_field_1.default)(this.bodyFields, fileName, file);
        })
            .catch(error => {
            this.rejectBadRequest(error);
        });
        this.fileStorePromises.push(processFile);
    }
    async processFile(fileName, stream, meta) {
        return new Promise((resolve, reject) => {
            stream.pipe((0, concat_stream_1.default)({ encoding: "buffer" }, async (buffer) => {
                const fileTypeFromBuffer = await file_type_1.default.fromBuffer(buffer);
                const fileTypeFromFileName = meta.filename.split(".").pop();
                const fileExtension = (fileTypeFromBuffer?.ext || fileTypeFromFileName) ?? "unknown";
                const mimeType = fileTypeFromBuffer?.mime ?? "application/octet-stream";
                if (this.options &&
                    this.options.options &&
                    this.options.options.extensions &&
                    this.options.options.extensions.length &&
                    !this.options.options.extensions.includes(fileExtension)) {
                    this.rejectBadRequest(`Sorry, ${fileExtension} files are not allowed`);
                }
                const factory = file_processor_factory_1.FileProcessorFactory.create({
                    fieldName: fileName.replace(/\[.*\]/, ""),
                    originalName: this.options.namingStrategy === "original"
                        ? meta.filename
                        : `${(0, crypto_1.randomUUID)()}.${fileExtension}`,
                    encoding: meta.encoding,
                    mimeType,
                    extension: fileExtension,
                    buffer,
                    config: {
                        provider: this.options.provider,
                        ...this.options.options,
                    },
                }, this.options);
                return resolve(factory.process());
            }));
        });
    }
    async processFileOnFinish() {
        await Promise.all(this.fileStorePromises);
        this.handleSuccess();
        this.handleFunctionToResolve(this.bodyFields);
    }
    handleSuccess() {
        this.request.unpipe(this.instance);
        this.instance.removeAllListeners();
    }
    rejectBadRequest(message) {
        const error = new common_1.BadRequestException(null, message);
        this.handleFunctionToReject(error);
        this.handleSuccess();
    }
}
exports.FormProcessor = FormProcessor;
